package svlight;

import java.util.*;
import java.awt.*;
import java.io.*;

import svlight.kernel.*;
//import sv.ui.SVUIC;

/** 
 * FrameManger.java
 * Manage the visualization frames
 * @version 1.0   10/1/97 
 * @author Byeongseob Ki
 */

public class FrameManager {

  private static SVClient svServer;

  // GFrame vectors
  private static Vector frame1d_vec;  
  private static Vector frame2d_vec;  

  // toggle for global command
  private boolean isGlobalCommand = false;   

  public FrameManager(SVClient svServer) {
     this.svServer = svServer;

     frame1d_vec = new Vector();
     frame2d_vec = new Vector();
  }


  /**
   * global command checking
   */
  public void clickedGlobalCheckButton(boolean b) {

  }

  /**
   * set drawingmode for each visualization frame
   */
  public void setDrawingMode(boolean b) {
  }


  /** 
   * register data set to this system
   * @param data data set to visualize
   * @param id expected id
   * @param c configurations
   */
  
  public long register(DataSet dataset, long id, Configs c) {
     long fid = -1;
     int dim = dataset.getRank();

     switch(dim) {
        case DataSet.ONED:
           fid = register_1d(dataset.getTitle(), dataset, id, (CPlane)c);
           break;
        case DataSet.TWOD:
           fid = register_2d(dataset.getTitle(), dataset, id, (Config2D)c);
           break;
        default:
           System.out.println("Unknown data set");
     }
     return fid;
  }

  /** 
   * received data set from the clients
   * @param data received data set
   * @param c configurations
   * @param id frame id
   * @param b if true, broadcast data set
   */
  public void receivedDataSet(DataSet data, Configs c, 
                                              long id, boolean b) {
     int dim = data.getRank();

     switch(dim) {
        case DataSet.ONED:
           received_1d(data, (CPlane)c, id, b);
           break;

        case DataSet.TWOD:
           received_2d(data, (Config2D)c, id, b);
           break;

        default:
           System.out.println("Unknown data set");
     }
  }   


  /**
   * action for 1d frame window
   * @param gf1d source GFPlane
   * @param id command id
   * @param args arguments
   * @param b if true, broadcast
   */
  public void action1d(GFPlane gf1d, int id, String[] args, boolean b) {
    switch(id) {
       case CommandSet.QUIT: 
         if (isGlobalCommand) {
           while (!frame1d_vec.isEmpty()) {
              GFPlane gframe = (GFPlane)frame1d_vec.lastElement();
              gframe.killFrame();
           }
           svServer.message("Killed all 1d windows.", false);

         } else 
           gf1d.killFrame();

         break;

       case CommandSet.ANIMATE:
          if (isGlobalCommand) {
           for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
              GFPlane gframe = (GFPlane)e.nextElement();
              gframe.doAnimate();
	   }
         } else 
           gf1d.doAnimate();

         break;

       case CommandSet.STOP:
          int time = Integer.valueOf(args[0]).intValue();
          if (isGlobalCommand) {
           for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
              GFPlane gframe = (GFPlane)e.nextElement();
              gframe.doStop();
              if (time >= gframe.noOfTime)
                time = gframe.noOfTime-1;
              gframe.currentTime = time;
	   }
	  } else {
            gf1d.doStop();
            if (time >= gf1d.noOfTime)
              time = gf1d.noOfTime-1;
            gf1d.currentTime = time;
          }

         break;

       case CommandSet.NEXT: 
          if (isGlobalCommand) {
           for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
              GFPlane gframe = (GFPlane)e.nextElement();
              gframe.next(true);
	   }
         } else 
           gf1d.next(true);

         break;

       case CommandSet.PREVIOUS: 
          if (isGlobalCommand) {
           for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
              GFPlane gframe = (GFPlane)e.nextElement();
              gframe.previous();
	   }
         } else 
           gf1d.previous();

         break;

       case CSPlane.CONNECTPOINT:
       case CSPlane.DISPLAYTIME:
        
          if (isGlobalCommand) {
           for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
              GFPlane gframe = (GFPlane)e.nextElement();
              gframe.doAction_1(id, args[0]);
	   }
         } else 
           gf1d.doAction_1(id, args[0]);

         break;

    }

    //    if (b)
    //      svServer.cmanager.broadcastCommand(gf1d, id, args);
  }


  /**
   * action for 2d frame window
   * @param gf2d source GFrame2D
   * @param id command id
   * @param args arguments
   * @param b if true, broadcast
   */
  public void action2d(GFrame2D gf2d, int id, String[] args, boolean b) {

    switch(id) {
       case CommandSet2D.QUIT: 
         if (isGlobalCommand) {
           while (!frame2d_vec.isEmpty()) {
              GFrame2D gframe = (GFrame2D)frame2d_vec.lastElement();
              gframe.killFrame();
           }
           svServer.message("Killed all 2d windows.", false);

         } else 
           gf2d.killFrame();

         break;

       case CommandSet2D.ANIMATE:
          if (isGlobalCommand) {
           for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
              GFrame2D gframe = (GFrame2D)e.nextElement();
              gframe.doAnimate();
	   }
         } else 
           gf2d.doAnimate();

         break;

       case CommandSet2D.STOP:
          int time = Integer.valueOf(args[0]).intValue();
          if (isGlobalCommand) {
           for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
              GFrame2D gframe = (GFrame2D)e.nextElement();
              gframe.doStop();
              if (time >= gframe.noOfTime)
                 time = gframe.noOfTime-1;
              gframe.currentTime = time;
	   }
          } else {
           gf2d.doStop();
           if (time >= gf2d.noOfTime)
              time = gf2d.noOfTime-1;
           gf2d.currentTime = time;
         }

         break;

       case CommandSet2D.NEXT: 
          if (isGlobalCommand) {
           for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
              GFrame2D gframe = (GFrame2D)e.nextElement();
              gframe.next(true);
	   }
         } else 
           gf2d.next(true);

         break;

       case CommandSet2D.PREVIOUS: 
          if (isGlobalCommand) {
           for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
              GFrame2D gframe = (GFrame2D)e.nextElement();
              gframe.previous();
	   }
         } else 
           gf2d.previous();

         break;

       case CommandSet2D.RESET:
          if (isGlobalCommand) {
           for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
              GFrame2D gframe = (GFrame2D)e.nextElement();
              gframe.canvas.reset();
	   }
         } else 
           gf2d.canvas.reset();

         break;

       case CommandSet2D.BOUNDBOX:
       case CommandSet2D.XYMESH:
       case CommandSet2D.FACEGRID:
       case CommandSet2D.VECTOR:
       case CommandSet2D.XYTICK:
       case CommandSet2D.ZTICK:
       case CommandSet2D.GMINMAX:
       case CommandSet2D.REGENERATE:
       case CommandSet2D.FILTEROVERLAY:
       case CommandSet2D.FILTERDISCARD:
       case CommandSet2D.DISPLAYTIME:
          if (isGlobalCommand) {
           for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
              GFrame2D gframe = (GFrame2D)e.nextElement();
              gframe.doAction_1(id, args[0]);
	   }
         } else 
           gf2d.doAction_1(id, args[0]);

         break;

       case CommandSet2D.WIRE:
       case CommandSet2D.WIRECOLOR:
       case CommandSet2D.HIDDEN:
       case CommandSet2D.COLOR:
       case CommandSet2D.GRAY:
       case CommandSet2D.POINTPLOT:
       case CommandSet2D.SURFACE:
       case CommandSet2D.CONTOUR:
          if (isGlobalCommand) {
           for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
              GFrame2D gframe = (GFrame2D)e.nextElement();
              gframe.doAction_2(id);
	   }
         } else 
           gf2d.doAction_2(id);

         break;

    }

    //    if (b)
    //      svServer.cmanager.broadcastCommand(gf2d, id, args);
  }


  /**
   * action for global variables
   * @param id command id
   * @param args arguments
   * @param b if true, broadcast
   */
  public void actionGlobal(int id, String[] args, boolean b) {
  }


  /**
   * action for filter
   * @param id frame id
   * @param dim frame dimension
   * @param Filter filter to be performed
   * @param b if true, broadcast
   */
  public void actionFilter(long id, int dim, String[] rids, 
                                Filter filter, String input, boolean b) {

  }
    

  /**
   * remove GFrame from the server
   * @param id DataSet id
   * @param dim dimension of the Graph Frame(1, 2, 3)
   */
  public void removeNotify(long id, int dim) {
    if (dim == DataSet.ONED) {
       for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
          GFPlane gframe = (GFPlane)e.nextElement();
          if (gframe.getID() == id) {
             frame1d_vec.removeElement(gframe);
             gframe = null;

             return;
          }
       }
    } else if (dim == DataSet.TWOD) {
       for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
          GFrame2D gframe = (GFrame2D)e.nextElement();
          if (gframe.getID() == id) {
             frame2d_vec.removeElement(gframe);
             gframe = null;

             return;
          }
       }
    } 
  }

  /**
   * return GFrame object from id
   * @param id GFrame id
   */
  public Object getGFrame(long id) {

     for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
        GFPlane gf1d = (GFPlane)e.nextElement();

        if (gf1d.getID() == id) {
           return gf1d;
        }
     }

     for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
        GFrame2D gf2d = (GFrame2D)e.nextElement();

        if (gf2d.getID() == id) {
           return gf2d;
        }
     }

     return null;
  }



  /**
   * return 1d frames
   */
  public Vector get1dFrames() {
     return frame1d_vec;
  }

  /**
   * return 2d frames
   */
  public Vector get2dFrames() {
     return frame2d_vec;
  }


  /**
   * return ith GFPlane
   */
  public GFPlane get1dFrame(int i) {
     return (GFPlane)frame1d_vec.elementAt(i);
  }


  /**
   * return ith GFrame2D
   */
  public GFrame2D get2dFrame(int i) {
     return (GFrame2D)frame2d_vec.elementAt(i);
  }


  /**
   * return frame1d_vec size
   */
  public int get1dvecSize() {
     return frame1d_vec.size();
  }

  /**
   * return frame2d_vec size
   */
  public int get2dvecSize() {
     return frame2d_vec.size();
  }


  /**
   * kill all frames
   * @param b if true, broadcast
   */
  public void killAllFrames(boolean b) {
  }


  // received 1d data set
  private synchronized void received_1d(DataSet data, CPlane c,
                                             long id, boolean b) {
      boolean endProcess = false;
      String title = data.getTitle();
      
      for (Enumeration e=frame1d_vec.elements(); e.hasMoreElements();) {
         GFPlane gframe = (GFPlane)e.nextElement();

         if (endProcess) break;
         for (int i=0; i<gframe.dataSetVector.size(); i++) {
            DSPlane dataset = (DSPlane)gframe.dataSetVector.elementAt(i);

            if (title.equals(dataset.getTitle()) && 
     ((DSPlane)data).getDataType() == ((DSPlane)dataset).getDataType()) {
                for (int j=0; j<data.getnoOfTime(); j++) {
                    TDPlane td = ((DSPlane)data).timeData[j];
                    boolean done = false;

                    for (int k=0; k<dataset.getnoOfTime(); k++) {
		      if (td.getTime() == dataset.getTimeValue(k)) {
                        if (id == 0)
                           dataset.updateTimeData(td, k, true);
                        else
                           dataset.updateTimeData(td, k, false);
                        gframe.updateDataSet();
                        done = true;
                        break;
                      }
                    }

                    if (!done) {
                      dataset.addTimeData(td);
                      gframe.updateDataSet(i, dataset);
	            }
                }
                endProcess = true;
		//                if (b)
		//                  svServer.cmanager.broadcastDataSet(data, gframe.getID());

                break;
            } 
         }
      }

      if (!endProcess) {
         long frameID = register_1d(title, data, id, c);    
	 //         if (b)
	 //            svServer.cmanager.broadcastDataSet(data, frameID);
      }
  }

  private synchronized void received_2d(DataSet data, Config2D c,
                                             long id, boolean b) {     
      boolean endProcess = false;
      String title = data.getTitle();
      boolean done = false;

      for (Enumeration e=frame2d_vec.elements(); e.hasMoreElements();) {
         GFrame2D gframe = (GFrame2D)e.nextElement();

         if (endProcess) break;
         for (int i=0; i<gframe.dataSetVector.size(); i++) {
            DataSet2D dataset = (DataSet2D)gframe.dataSetVector.elementAt(i);

            if (title.equals(dataset.getTitle())) {
                for (int j=0; j<data.getnoOfTime(); j++) {
                    TimeData2D td = ((DataSet2D)data).timeData[j];

                    for (int k=0; k<dataset.getnoOfTime(); k++) {
		      if (td.getTime() == dataset.getTimeValue(k)) {
                        dataset.updateTimeData(td, k);
                        gframe.updateDataSet();
                        done = true;
                        break;
                      }
                    }

                    if (!done) {
                      dataset.addTimeData(td);
                      gframe.updateDataSet(i, dataset);
	            }
                }
                endProcess = true;

		//                if (b)
		//                  svServer.cmanager.broadcastDataSet(data, gframe.getID());
                break;
            } 
         }
      }

      if (!endProcess) {
         long frameID = register_2d(title, data, id, c);    
	 //         if (b)
	 //            svServer.cmanager.broadcastDataSet(data, frameID);
      }
  }


  private synchronized long register_1d(String title, DataSet dataset, 
                                             long id, CPlane c) {
     if (c == null) {
        c = new CPlane();
     }

     int type = ((DSPlane)dataset).getDataType();
   
     GFPlane gframe;
     if (type == DSPlane.NORMAL)
       gframe = new GFPOneD(title, (DataSet1D)dataset, c, svServer);
     //     else if (type == DSPlane.VECTOR)
     //       gframe = new GFPVec(title, (DataSetVec)dataset, c, svServer);  
     //     else if (type == DSPlane.CONNECTIVITY)
     //       gframe = new GFPConnect(title, (DataSetCon)dataset, c, svServer);  
     else 
       return -1;

     frame1d_vec.addElement(gframe);
     if (id > 0)
        gframe.setID(id);

     gframe.setSize(400, 400);
     gframe.show();    

     return gframe.getID();
  }

  private synchronized long register_2d(String title, DataSet dataset, 
                                              long id, Config2D c) {

     if (c == null) {
        c = new Config2D();
     }

     GFrame2D gframe = new GFrame2D(title, (DataSet2D)dataset, c, svServer);
     frame2d_vec.addElement(gframe);
     if (id > 0)
        gframe.setID(id);

     gframe.setSize(400, 400);
     gframe.show();    

     return gframe.getID();
  }

} // end of class







