package svlight;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import svlight.kernel.*;
import svlight.ui.*;
import svlight.util.*;
import svlight.drawer.*;

/** 
 * GCPlane.java
 * Define a canvas to draw the graph
 * @version 1.0    10/15/97
 * @author Byeongseob Ki
 */

abstract class GCPlane extends GCanvas {

  // size of the canvas
  protected Dimension canvasSize;

  // canvas size of point data area 
  protected Rectangle pda;

  // axis
  protected Axis axis;

  // min and max value of given time slot
  protected float[] minmaxArr;

  // dragging
  protected boolean inversedX = false;
  protected boolean inversedY = false;
  protected int dragStartX, dragStartY, dragEndX, dragEndY;

  protected boolean isPlotAreaResize = false;
  protected int isLabelMove = -1;
  protected int selectedCorner;
  protected Rectangle pdaCorners[];

  // zooming
  protected float zoomFactorX = 1.0f, zoomFactorY = 1.0f;
  protected int zoomMode = 3;         // zoom mode 0 - xy zoom
                            //           1 - x  zoom
                            //           2 - y  zoom 
                            //           3 - no zoom 
  protected Vector zoomer;

  // to access GFPlane class
  protected GFPlane gframe;

  // to access svserver
  protected SVClient svServer;
  
   // resizing
  protected float resizeFactor = 1.0f;
  protected int fontSize = 12;

  // labels
  protected RTextLine text;

  // number format
  public int xsig = 3; 
  public int ysig = 3; 
  public int timesig = 3; 
  public int xprec   = 2; 
  public int yprec   = 2; 
  public int timeprec   = 2; 
  public boolean sformat = false;


  /**
   * constructor
   */
  public GCPlane(GFPlane gframe) {
     this.gframe = gframe;
     this.svServer = gframe.svServer;

     text = new RTextLine();
     text.setColor(Color.black);
     text.setDrawingComponent(this);

     pda = new Rectangle();
     pdaCorners = new Rectangle[2];
     rect = new Rectangle();
     minmaxArr = gframe.getMinMax();
     axis = new Axis();

     makepopup();

     zoomer = new Vector();
     zoomFactorX = gframe.config.zoomFactorX;
     zoomFactorY = gframe.config.zoomFactorY;
     zoomMode = gframe.config.zoomMode;

  }  

  /**
   * return offscreen image
   */
  public Image getImage() {
     return offScrImage;
  }

  /**
   * set data plot area
   */
  public void setPDA(Rectangle rect) {
     pda = rect;
     change_pdaCorners();
  }

  /**
   * notify the size of canvas to all attached elements
   * to recalculate the data and repaint it
   */
  public void changeNotify() {
     changeNotify(getSize());
  }

  // implementation of changeNotify
  protected abstract void changeNotify(Dimension d);



  // collaboration zooming
  public void colZooms(String[] args) {
    minmaxArr[0] = Float.valueOf(args[0]).floatValue();
    minmaxArr[1] = Float.valueOf(args[1]).floatValue();
    minmaxArr[2] = Float.valueOf(args[2]).floatValue();
    minmaxArr[3] = Float.valueOf(args[3]).floatValue();

    zoomFactorX = Float.valueOf(args[4]).floatValue();
    zoomFactorY = Float.valueOf(args[5]).floatValue();
    zoomMode = Integer.valueOf(args[6]).intValue();

    gframe.config.zoomStartX = minmaxArr[0];
    gframe.config.zoomStartY = minmaxArr[1];
    gframe.config.zoomEndX = minmaxArr[2];
    gframe.config.zoomEndY = minmaxArr[3];
    gframe.config.zoomFactorX = zoomFactorX;
    gframe.config.zoomFactorY = zoomFactorY;
    gframe.config.zoomMode = zoomMode;

    zoomer.addElement(new ZoomHandle(minmaxArr[0], minmaxArr[1],
         minmaxArr[2], minmaxArr[3], zoomFactorX, zoomFactorY, zoomMode));

    repaint();
  }


  public void resetZooming() {
     while (!zoomer.isEmpty()) {
        ZoomHandle zh = (ZoomHandle)zoomer.lastElement();
        zoomer.removeElement(zh);
        zh = null;
     }
     gframe.zoomoutMI.setEnabled(false);
     zoomFactorX = 1.0f;
     zoomFactorY = 1.0f;
     minmaxArr = gframe.getMinMax();
     repaint();
  }

  public void zoomOut() {
    ZoomHandle zh;

    if (!zoomer.isEmpty()) {
        zh = (ZoomHandle)zoomer.lastElement();
        zoomer.removeElement(zh);

    } else {
        gframe.zoomoutMI.setEnabled(false);
        return;
    }

    String args[] = zh.getAllInfo();

    minmaxArr[0] = Float.valueOf(args[0]).floatValue();
    minmaxArr[1] = Float.valueOf(args[1]).floatValue();
    minmaxArr[2] = Float.valueOf(args[2]).floatValue();
    minmaxArr[3] = Float.valueOf(args[3]).floatValue();

    zoomFactorX = Float.valueOf(args[4]).floatValue();
    zoomFactorY = Float.valueOf(args[5]).floatValue();
    zoomMode = Integer.valueOf(args[6]).intValue();

    gframe.config.zoomStartX = minmaxArr[0];
    gframe.config.zoomStartY = minmaxArr[1];
    gframe.config.zoomEndX = minmaxArr[2];
    gframe.config.zoomEndY = minmaxArr[3];
    gframe.config.zoomFactorX = zoomFactorX;
    gframe.config.zoomFactorY = zoomFactorY;
    gframe.config.zoomMode = zoomMode;

    if (zoomer.isEmpty()) 
       gframe.zoomoutMI.setEnabled(false);

    zh = null;
    repaint();
  }



  // draw objects
  protected void objectsDraw(Graphics g) {
  }

  // find PDA coners
  protected void change_pdaCorners() {
     pdaCorners[0] = new Rectangle(pda.x-10, pda.y-10, 20, 20);
     pdaCorners[1] = 
         new Rectangle(pda.x+pda.width-10, pda.y+pda.height-10, 20, 20);
  }

  // Find the closest data point to the cursor
  protected float[] getClosestPoint(float x, float y) {
     float a[] = new float[3];
     float distsq = -1.0f;
     float data[] = {0.0f, 0.0f};

     for (int i=0; i<gframe.dataSetVector.size(); i++) {
        DataSet1D ds = (DataSet1D)gframe.dataSetVector.elementAt(i);

        for (int j=0; j<ds.getTimeArr().length; j++) {
           if (ds.getTimeValue(j) == gframe.timeArr[gframe.currentTime]) {

               a = ds.timeData[j].getClosestPoint(x,y);

               if(distsq < 0.0f || distsq > a[2]) {
                    data[0] = a[0];
                    data[1] = a[1];
                    distsq  = a[2];
	        }
	    }
         }
     }
     return data;
  }

  /**
   * do whiteboard related work
   */
  public void doWhiteboardWork(int id) {
  }

  /**
   * do add point action
   */
  public void doWAddPoint(int type, int x, int y) {
  }


  /**
   * do add points action
   */
  public void doWAddPoints(String[] args) {
  }



  // make popup menu for the whiteboard
  protected void makepopup() {
  }

} // end of class
