package svlight;

import java.awt.*;
import java.util.*;
import java.io.*;
import java.awt.event.*;

import svlight.util.*;
import svlight.ui.*;
import svlight.drawer.*;
import svlight.kernel.*;

/** 
 * GFPOneD.java
 * Define a frame to draw the graph
 * @version 1.0    10/1/97
 * @author Byeongseob Ki
 */

class GFPOneD extends GFPlane {

  // various MenuItems
  private MenuItem lmoveMI;

  /**
   * constructor
   * @param title title of this class
   * @param dataSet dataSet for this class
   * @param svServer svserver
   */
  public GFPOneD(String title, DataSet1D dataSet, 
                            CPlane config, SVClient svServer) {
     super(title, dataSet, config, svServer);

     // layout others     
     setLayout(new BorderLayout());
     canvas = new GCPOneD(this);

     canvas.dh = new DrawHandle(timeArr[currentTime], canvas.getSize());
     DHV.addElement(canvas.dh);

     boolean b = false;

     if (config.colormap != null) {
         b = readColorMap(config.colormap);

         if (b) {
           float value;

           for (int i=0; i<101; i++) {
              value = rspline.computePoint((float)i/100f); 
              if (value > 1.0f) value = 1.0f;
              else if (value < 0.0f) value = 0.0f;
              rcolors[i] = value;

              value = gspline.computePoint((float)i/100f);    
              if (value > 1.0f) value = 1.0f;
              else if (value < 0.0f) value = 0.0f;
              gcolors[i] = value;

              value = bspline.computePoint((float)i/100f);    
              if (value > 1.0f) value = 1.0f;
              else if (value < 0.0f) value = 0.0f;
              bcolors[i] = value;
           }
         }
     }

     if (config.colormap == null || b == false) {
         int nControls = 5;
         float[] xcontrols = { 0.0f, 0.25f, 0.5f, 0.75f, 1.0f };
         float[] ycontrols = { 0.0f, 0.5f, 1.0f, 0.5f, 0.0f };
         rspline = new NewtonMethod(xcontrols, ycontrols);
         hspline = new NewtonMethod(xcontrols, ycontrols);

         for (int i=0; i<5; i++)
           ycontrols[i] = 0.5f;
         gspline = new NewtonMethod(xcontrols, ycontrols);
         sspline = new NewtonMethod(xcontrols, ycontrols);

         ycontrols[0] = 1.0f;
         ycontrols[1] = 0.5f; 
         ycontrols[2] = 0.0f;
         ycontrols[3] = 0.5f;
         ycontrols[4] = 1.0f;
         bspline = new NewtonMethod(xcontrols, ycontrols);
         lspline = new NewtonMethod(xcontrols, ycontrols);
     }

     add("Center", canvas);
     add("South", monitor = new SimpleMonitor());
     monitor.display(dataSet.getTitle() + " created.", false);
  }   


  /**
   * do additional work after get the user input
   * @param command command to do work
   * @param input user input
   */
  public void doWork(int command, String input) {
     super.doWork(command, input);

     if (command == CSPlane.ADDTIME) {
     }  else if (command == CSPlane.LEGENDCHANGE) {
     } else if (command == CSPlane.LEGENDMOVE) {
     } else if (command == CSPlane.LABELMOVE) {
     } 

  } 


  /**
   * action for boolean arguments
   */
  public void doAction_1(int id, String arg) {
     boolean b;
     if (arg.equals("true"))
        b = true;
     else
        b = false;

     switch (id) {
      case CSPlane.CONNECTPOINT:
          lineMI.setState(b);
          config.connect_point = lineMI.getState();
          break;

      case CSPlane.DISPLAYLBL:
          labelMI.setState(b);
          config.display_label = labelMI.getState();

          if (config.display_label) {
             clabelMI.setEnabled(true);
             labelmMI.setEnabled(true);
          } else {
             clabelMI.setEnabled(false);
             labelmMI.setEnabled(false);
          }
            
          canvas.changeNotify();
          break;

      case CSPlane.TICKMARK:
          tickerMI.setState(b);
          config.tickmark = tickerMI.getState();

          if (config.tickmark) 
             ctickerMI.setEnabled(true);
          else
             ctickerMI.setEnabled(false);

          canvas.changeNotify();
          break;

      case CSPlane.DISPLAYLGD:
          legendMI.setState(b);
          config.display_legend = legendMI.getState();
          lmoveMI.setEnabled(b);

          if (config.display_legend) 
             clegendMI.setEnabled(true);
          else
             clegendMI.setEnabled(false);

          canvas.changeNotify();
          break;

      case CSPlane.GMINMAX:
          checkMI.setState(b);
          config.global_mode = b;
          globalFlag = checkMI.getState();
          if (globalFlag)
              calcGlobalMinMax();
          canvas.minmaxArr = getMinMax();
          break;

      case CSPlane.AMR:
          amrMI.setState(b);
          if (amrMI.getState()) {
            for (Enumeration e = dataSetVector.elements(); 
                                             e.hasMoreElements(); ) {
               DataSet1D dataSet1D = (DataSet1D)e.nextElement();
               for (int j=0; j<dataSet1D.getnoOfTime(); j++)
                  ((TimeData1D)(dataSet1D.timeData[j])).calcAMRColor(colorModel, rcolors, gcolors, bcolors, hcolors, scolors, lcolors);
            }
          }
          config.amr = amrMI.getState();

          if (config.amr) {
             cinfoMI.setState(false);
             config.color_by_info = cinfoMI.getState();
             csetMI.setState(false);
             config.color_by_set = csetMI.getState();
          }

          break;

      case CSPlane.COLORBYINFO:
          cinfoMI.setState(b);
          if (cinfoMI.getState()) {
            for (Enumeration e = dataSetVector.elements(); 
                                             e.hasMoreElements(); ) {
               DataSet1D dataSet1D = (DataSet1D)e.nextElement();
               for (int j=0; j<dataSet1D.getnoOfTime(); j++)
                  ((TimeData1D)(dataSet1D.timeData[j])).calcColorByInfo(colorModel, rcolors, gcolors, bcolors, hcolors, scolors, lcolors);
            }
          }
          config.color_by_info = cinfoMI.getState();

          if (config.color_by_info) {
             amrMI.setState(false);
             config.amr = amrMI.getState();
             csetMI.setState(false);
             config.color_by_set = csetMI.getState();
          }
          break;

      case CSPlane.COLORBYSET:
          csetMI.setState(b);
 
          Color[] colorIndex = {
             Color.red, Color.blue, Color.magenta, Color.orange,
             Color.green, Color.pink, Color.yellow, Color.cyan
          };

          Color[] carrs= new Color[dataSetVector.size()];
          for (int i=0; i<carrs.length; i++) {
             if (i < colorIndex.length) 
                carrs[i] = colorIndex[i];
             else
                carrs[i] = Color.black;
          }

          int cidx = 0; 
          if (csetMI.getState()) {
            for (Enumeration e = dataSetVector.elements(); 
                                             e.hasMoreElements(); ) {
               DataSet1D dataSet1D = (DataSet1D)e.nextElement();
               dataSet1D.setLineColor(carrs[cidx++]);
            }
          }
          carrs = null;
          config.color_by_set = csetMI.getState();

          if (config.color_by_set) {
             amrMI.setState(false);
             config.amr = amrMI.getState();
             cinfoMI.setState(false);
             config.color_by_info = cinfoMI.getState();
          }
          break;

      case CSPlane.DISPLAYTIME:
          timeMI.setState(b);
          config.display_time = timeMI.getState();
          break;

      case CSPlane.FILTEROVERLAY:
          overlayMI.setState(b);
          config.overlay_mode = overlayMI.getState();
 
          if (config.overlay_mode) {
             monitor.display("Filter overlay mode is on.", true);
             discardMI.setState(false);
          } else {
             monitor.display("Filter overlay mode is off.", true);
             discardMI.setState(true);
          }
          config.discard_mode = discardMI.getState(); 
          return;

      case CSPlane.FILTERDISCARD:
          discardMI.setState(b);
          config.discard_mode = discardMI.getState();
 
          if (config.discard_mode) {
             monitor.display("Filter discard mode is on.", true);
             overlayMI.setState(false);
          } else {
             monitor.display("Filter discard mode is off.", true);
             overlayMI.setState(true);
          }
          config.overlay_mode = overlayMI.getState();
          return;


     }

     canvas.repaint();
  }

  public void doAction_2(int id, String args[]) {
     int x, y, w, h;

     switch (id) {
      case CSPlane.BACKCOLOR:
          int r = Integer.valueOf(args[0]).intValue();
          int g = Integer.valueOf(args[1]).intValue();
          int b = Integer.valueOf(args[2]).intValue();
          config.bgColor = new Color(r, g, b);
          break;

      case CSPlane.RESIZEPDA:
          x = Integer.valueOf(args[0]).intValue();
          y = Integer.valueOf(args[1]).intValue();
          w = Integer.valueOf(args[2]).intValue();
          h = Integer.valueOf(args[3]).intValue();
          canvas.setPDA(new Rectangle(x, y, w, h));
          break;

      case CSPlane.LEGENDMOVE:
          x = Integer.valueOf(args[0]).intValue();
          y = Integer.valueOf(args[1]).intValue();
          w = Integer.valueOf(args[2]).intValue();
          h = Integer.valueOf(args[3]).intValue();
          config.legendRect = new Rectangle(x, y, w, h);
          break;

      case CSPlane.LABELMOVE:
          config.xLabelPoint.x = Integer.valueOf(args[0]).intValue();
          config.xLabelPoint.y = Integer.valueOf(args[1]).intValue();
          config.yLabelPoint.x = Integer.valueOf(args[2]).intValue();
          config.yLabelPoint.y = Integer.valueOf(args[3]).intValue();
          break;

      case CSPlane.TICKERCHANGE:
          int ints[] = { 
              Integer.valueOf(args[0]).intValue(), 
              Integer.valueOf(args[1]).intValue(),  
              Integer.valueOf(args[2]).intValue(), 
              Integer.valueOf(args[3]).intValue(), 
              Integer.valueOf(args[4]).intValue(),
              Integer.valueOf(args[5]).intValue(), 
              Integer.valueOf(args[6]).intValue(), 
              Integer.valueOf(args[7]).intValue(), 
              Integer.valueOf(args[8]).intValue(), 
              Integer.valueOf(args[9]).intValue() };

          boolean bools[] = new boolean[2];
          if (args[10].equals("true"))
              bools[0] = true;
          else
              bools[0] = false;

          if (args[11].equals("true"))
              bools[1] = true;
          else
              bools[1] = false;

          canvas.axis.setSetting(ints, bools);
          break;
      }

      canvas.repaint();
  }

  /**
   * set color map
   */
  public void setColorMap(String[] str) {
     super.setColorMap(str);

     for (Enumeration e = dataSetVector.elements(); e.hasMoreElements(); ) {
        DataSet1D dataSet1D = (DataSet1D)e.nextElement();
        for (int j=0; j<dataSet1D.getnoOfTime(); j++) {
           if (config.amr)         
              ((TimeData1D)(dataSet1D.timeData[j])).calcAMRColor(colorModel, rcolors, gcolors, bcolors, hcolors, scolors, lcolors);   
           else if (config.color_by_info) 
              ((TimeData1D)(dataSet1D.timeData[j])).calcColorByInfo(colorModel, rcolors, gcolors, bcolors, hcolors, scolors, lcolors);

        }
     }

     canvas.repaint();
   }       

  /**
   * legend change action
   */
  public void doLegendChange(String args[]) {
  }

 
  // write all objects of this frame to a file
  public void writeToFile(String fname) {
  }


  // make menu system
  protected void makeMenu() {
     MenuBar menubar = new MenuBar();
     CSPlane command;
     MenuItem mi;
     int filter_command = 1000;

     // File manager
     Menu fileMenu = new Menu("File");

     addMI("Quit C-q", fileMenu, new CSPlane(this, CSPlane.QUIT));

     menubar.add(fileMenu);

     // Configuration
     Menu configMenu = new Menu("View");

     lineMI = new CheckboxMenuItem("Connect Points");
     command = new CSPlane(this, CSPlane.CONNECTPOINT);
     lineMI.addItemListener(command);
     lineMI.setState(config.connect_point); 
     configMenu.add(lineMI);

     timeMI = new CheckboxMenuItem("Display Time");
     command = new CSPlane(this, CSPlane.DISPLAYTIME);
     timeMI.addItemListener(command);
     timeMI.setState(config.display_time);
     configMenu.add(timeMI);

     menubar.add(configMenu);

     ////////////////////////////////////////////////////
     // FILTERS
     ////////////////////////////////////////////////////

     ////////////////////////////////////////////////////

     // Misc menu
     Menu miscMenu = new Menu("Misc");

     // animation menus
     Menu animationMenu = new Menu("Animation");

     animateMI = new MenuItem("Animate C-a");
     command = new CSPlane(this, CSPlane.ANIMATE);
     animateMI.addActionListener(command);
     animationMenu.add(animateMI);

     stopMI = new MenuItem("Stop C-s");
     command = new CSPlane(this, CSPlane.STOP);
     stopMI.addActionListener(command);
     animationMenu.add(stopMI);
     stopMI.setEnabled(false);

     nextMI = new MenuItem("Next C-n");
     command = new CSPlane(this, CSPlane.NEXT);
     nextMI.addActionListener(command);
     animationMenu.add(nextMI);

     prevMI = new MenuItem("Previous C-p");
     command = new CSPlane(this, CSPlane.PREVIOUS);
     prevMI.addActionListener(command);
     animationMenu.add(prevMI);
    
     miscMenu.add(animationMenu);

     menubar.add(miscMenu);

     setMenuBar(menubar);
  }

}  // end of class

