package svlight;

import java.awt.*;
import java.util.*;
import java.io.*;
import java.awt.event.*;

import svlight.util.*;
import svlight.ui.*;
import svlight.drawer.*;
import svlight.kernel.*;

/** 
 * GFPlane.java
 * Define a frame to draw the graph
 * @version 1.0    10/1/97
 * @author Byeongseob Ki
 */

public abstract class GFPlane extends GFrame  {

  // to access parent
  public SVClient svServer;

  // canvas to draw the graph
  public GCPlane canvas;


  // global min and max value
  public float globalxmin = Float.MAX_VALUE, globalxmax = Float.MIN_VALUE, 
               globalymin = Float.MAX_VALUE, globalymax = Float.MIN_VALUE;


  // animation class
  protected Animator animator;

  // Thread to creat GIF image
  //  protected GIFer gifer;

  // CutCopy manager dialog
  //  protected CutCopyManager editManager;

  // animation speed
  protected int animationSpeed = 10;

  // for merging
  int mergeIndex = 1;
  boolean mergeSelected = false;

  // various MenuItems
  protected MenuItem animateMI, stopMI, nextMI, prevMI, splitMI, labelmMI;
  public CheckboxMenuItem checkMI, tickerMI, labelMI, amrMI, lineMI, legendMI,
                    timeMI, stealthMI, overlayMI, discardMI, cinfoMI, csetMI;
  public MenuItem zoomoutMI, ctickerMI, clabelMI, clegendMI;

  // configuration
  public CPlane config;

  // data type of this window frame
  protected int dataType;

  /**
   * constructor
   * @param title title of this class
   * @param dataSet dataSet for this class
   * @param svServer svserver
   */
  public GFPlane(String title, DSPlane dataSet, 
                            CPlane config, SVClient svServer) {
     super(title);

     this.svServer = svServer;
     dimension_ = DataSet.ONED;
     this.config = config;
     dataType = dataSet.getDataType();

     // copy the DataSet
     dataSetVector.addElement(dataSet);

     makeMenu();

     // initialize timeArr
     noOfTime = dataSet.getnoOfTime();
     timeArr = dataSet.getTimeArr();
     calcGlobalMinMax();
  
     // drawing handle vector
     DHV = new Vector();

  }   

  /** 
   * return data type
   */
  public int getDataType() {
     return dataType;
  }


  /**
   * calculate global min and max values for x and y
   * and update each time data
   */
  public void calcGlobalMinMax() {
    Enumeration e;
 
    globalxmin = Float.MAX_VALUE;
    globalxmax = Float.MIN_VALUE;
    globalymin = Float.MAX_VALUE;
    globalymax = Float.MIN_VALUE;

    for (e=dataSetVector.elements(); e.hasMoreElements();) {
       DSPlane data = (DSPlane)e.nextElement();
       for (int i=0; i<data.getnoOfTime(); i++) {
          if (globalxmin > data.timeData[i].getxmin())
             globalxmin = data.timeData[i].getxmin();
          if (globalymin > data.timeData[i].getymin())
             globalymin = data.timeData[i].getymin();
          if (globalxmax < data.timeData[i].getxmax())
             globalxmax = data.timeData[i].getxmax();
          if (globalymax < data.timeData[i].getymax())
             globalymax = data.timeData[i].getymax();
       }
    }
    
    for (e=dataSetVector.elements(); e.hasMoreElements();) {
       DSPlane data = (DSPlane)e.nextElement();
       for (int i=0; i<data.getnoOfTime(); i++) 
           data.timeData[i].setGlobalValues(globalxmin, globalxmax,
                                            globalymin, globalymax);
    }

    if (canvas != null)
       canvas.minmaxArr = getMinMax();
  }


  /**
   * add another dataSet to this frame
   * @param dataSet DSPlane to be added
   */
  public void addDataSet(DSPlane dataSet) {
     // reset all the currentTime value to 0
     currentTime = 0;

     dataSetVector.addElement(dataSet);
     updateTimeArr();
     calcGlobalMinMax();
     splitMI.setEnabled(true);

     monitor.display("Added new DSPlane " + dataSet.getTitle(), false);
     setFrameTitle(dataSet.getTitle() + "...");
     canvas.repaint();
  }


  /**
   * update dataSet to this frame
   * @param pos position of DataSet to be replaced
   * @param dataSet DSPlane to be updated
   */
  public void updateDataSet(int pos, DSPlane dataSet) {

     dataSetVector.removeElementAt(pos);
     dataSetVector.addElement(dataSet);
     updateTimeArr();

     calcGlobalMinMax();

     if (currentTime != noOfTime-1) {
        int saved = currentTime;
        currentTime = 0;

        // send to last time
        for (int i=0; i<saved; i++)
           next(false);
        next(true);   // one more time
     }

     monitor.display("Updated DSPlane " + dataSet.getTitle(), false);
     canvas.repaint();
  }

  /**
   * update dataSets to this frame
   */
  public void updateDataSet() {

     calcGlobalMinMax();
     canvas.repaint();
  }


  /**
   * remove d ataSet from this frame
   * @param str DSPlane title to be removed
   */
  public void removeDataSet(String str) {
     
    for (Enumeration e=dataSetVector.elements(); e.hasMoreElements();) {
       DSPlane data = (DSPlane)e.nextElement();

       if (str.equals(data.getTitle())) {
          removeDataSet(data);
          break;
       }
    } 
  }

  /**
   * remove a dataSet from this frame
   * @param dataSet DSPlane to be removed
   */
  public void removeDataSet(DSPlane dataSet) {
     dataSetVector.removeElement(dataSet);

     // reset all the currentTime value to 0
     currentTime = 0;

     if (dataSetVector.size() <= 0) {
        svServer.fmanager.removeNotify(id, DataSet.ONED);
        this.dispose();
     } else {
         DSPlane data = (DSPlane)dataSetVector.firstElement();
         if (dataSetVector.size() == 1) {
            setFrameTitle(data.getTitle());
            splitMI.setEnabled(false);
         } else
            setFrameTitle(data.getTitle() + "...");
     }

     updateTimeArr();
     calcGlobalMinMax();

     monitor.display("Removed DSPlane " + dataSet.getTitle(), false);
     canvas.repaint();
  }



  /** 
   * calculate xmin, xmax, ymin, and xmax for each time slot
   * @return minmax array of min and max
   */
  public float[] getMinMax() {

    Enumeration e;
    float minmaxArr[] = new float[4];   

    minmaxArr[0] = 1.0e10f;
    minmaxArr[1] = -1.0e10f;
    minmaxArr[2] = 1.0e10f;
    minmaxArr[3] = -1.0e10f;

    if (globalFlag) {
      minmaxArr[0] = globalxmin;
      minmaxArr[1] = globalxmax;
      minmaxArr[2] = globalymin;
      minmaxArr[3] = globalymax;
      return minmaxArr;
    }

    for (e=dataSetVector.elements(); e.hasMoreElements();) {
       DSPlane data = (DSPlane)e.nextElement();
       
       for (int i=0; i<data.getTimeArr().length; i++) {
          if (timeArr[currentTime] == data.getTimeValue(i)) {
             if (minmaxArr[0] > data.timeData[i].getxmin()) 
                minmaxArr[0] =  data.timeData[i].getxmin();
             if (minmaxArr[1] < data.timeData[i].getxmax()) 
                minmaxArr[1] =  data.timeData[i].getxmax();
             if (minmaxArr[2] > data.timeData[i].getymin()) 
                minmaxArr[2] =  data.timeData[i].getymin();
             if (minmaxArr[3] < data.timeData[i].getymax()) 
                minmaxArr[3] =  data.timeData[i].getymax();
             break;
          }
       }
    }
    return minmaxArr;    
  }



  /**
   * set color map
   */
  public void setColorMap(String[] str) {
     super.setColorMap(str);
  }



  /**
   * internal merge action
   * @param b if control key is pressed
   */
  public void mergeAction(boolean b) {
  }


  /**
   * do additional work after get the user input
   * @param command command to do work
   * @param input user input
   */
  public void doWork(int command, String input) {
    if (command == CSPlane.SAVEAS) {
    } else if (command == CSPlane.PRINT) {
    } else if (command == CSPlane.GIFSEQUENCE) {
    } else if (command == CSPlane.CUT) {
    } else if (command == CSPlane.COPY) {
    } else if (command == CSPlane.PASTE) {
    } else if (command == CSPlane.COLORMAP) {
    } else if (command == CSPlane.XYZOOM) {
    } else if (command == CSPlane.XZOOM) {
    } else if (command == CSPlane.YZOOM) {
    } else if (command == CSPlane.NOZOOM) {
    }  else if (command == CSPlane.GOTO) {
    } else if (command == CSPlane.RESIZEPDA) {
    } else if (command == CSPlane.RENAMETITLE) {
    } else if (command == CSPlane.ASPEEDCHANGE) {
    } else if (command == CSPlane.LABELCHANGE) {
    } else if (command == CSPlane.TICKERCHANGE) {
    } else if (command == CSPlane.STEALTH) {
    } else if (command == CSPlane.BUFFERSIZE) {
    }
  }



  /**
   * kill this frame
   */
  public void killFrame() {

      if (animator != null) {
        animator.stop();
        animator = null;
      } 

      dataSetVector.removeAllElements();
      dataSetVector = null;

      svServer.fmanager.removeNotify(id, DataSet.ONED);
      this.dispose();
   }


  // Cut and copy command
  protected void doCutCopy(boolean cutting) {
  }

  /**
   * do multiple cut and copy
   */
  public void multiCutCopy(int[] selected, boolean cutting) {
  }

  /**
   * do animation command
   */
  public void doAnimate() {
     if (animator == null) {
        animator = new Animator(this, animationSpeed);
        animator.start();
     } else 
        System.out.println("Animation thread was corrupted ...");

     animateMI.setEnabled(false);
     nextMI.setEnabled(false);
     prevMI.setEnabled(false);
     stopMI.setEnabled(true);
  
  }

  /**
   * do animation  stop command
   */
  public void doStop() {
     if (animator != null) {
       animator.stop();
       animator = null;
     }
     animateMI.setEnabled(true);
     stopMI.setEnabled(false);
     nextMI.setEnabled(true);
     prevMI.setEnabled(true);
  }

  /** 
   * get next DataSet
   * @param done if true, repaint canvas
   */
  public void next(boolean done) {

    currentTime++;
    if (currentTime == noOfTime)
       currentTime = 0;

    if (done) {
       boolean endprocess = false;
       for (Enumeration e=DHV.elements(); e.hasMoreElements();) {
          DrawHandle dh = (DrawHandle)e.nextElement();
          if (dh.getTime() == timeArr[currentTime]) {
             canvas.dh = dh;
             endprocess = true;
             break;
          }
       }
       if (!endprocess) {
          canvas.dh = new DrawHandle(timeArr[currentTime], canvas.getSize());
          DHV.addElement(canvas.dh);
       }

       canvas.minmaxArr = getMinMax();
       canvas.repaint();
    }
  }
  
  /** 
   * get Previous DataSet (for Animation)
   */
  public void previous() {

    currentTime--;
    if (currentTime == -1)
       currentTime = noOfTime-1;

    boolean endprocess = false;
    for (Enumeration e=DHV.elements(); e.hasMoreElements();) {
       DrawHandle dh = (DrawHandle)e.nextElement();
       if (dh.getTime() == timeArr[currentTime]) {
          canvas.dh = dh;
          endprocess = true;
          break;
       }
    }
    if (!endprocess) {
       canvas.dh = new DrawHandle(timeArr[currentTime], canvas.getSize());
       DHV.addElement(canvas.dh);
    } 
    canvas.minmaxArr = getMinMax();
    canvas.repaint();
  }


  /**
   * change color map info
   */
  public void changeColorMap(String[] args) {
     svServer.fmanager.action1d(this, CommandSet.COLORMAPCHANGE, args, true);
  }
  

  /**
   * action for boolean arguments
   */
  public abstract void doAction_1(int id, String arg);

  /**
   * action for multiple arguments
   */
  public abstract void doAction_2(int id, String args[]);

  /** 
   * split command
   */
  public void doSplit() {

     DSPlane data = (DSPlane)dataSetVector.lastElement();

     long frameID  = svServer.fmanager.register(data, -1, null);
     removeDataSet(data);

     canvas.repaint();
  }


  /**
   * label change action
   */
  public void doLabelChange(String args[]) {
      config.xLabel = args[0];
      config.yLabel = args[1];
      config.xLabelFont = new Font(args[2],
                       Integer.valueOf(args[3]).intValue(),
                       Integer.valueOf(args[4]).intValue());
      config.yLabelFont = new Font(args[5],
                       Integer.valueOf(args[6]).intValue(),
                       Integer.valueOf(args[7]).intValue());

      if (args[8].equals("true"))
         config.isXLabelRotate = true;
      else
         config.isXLabelRotate = false;

      if (args[9].equals("true"))
         config.isYLabelRotate = true;
      else
         config.isYLabelRotate = false;

      canvas.repaint();
  }

  /**
   * goto action
   */
  public void doGotoAction(String input) {
      int whereto = Integer.valueOf(input.trim()).intValue();

      if ((whereto > noOfTime) || (whereto <= 0))
         monitor.display("Invalid range ...", true);

      int index = 0;
      while (whereto != currentTime) {
         next(false);

         if (index > noOfTime) break;
         index++;
       }
      previous();
  }


  /**
   * buffer size change action
   */
  public void doBufferAction(String input) {
      int size = Integer.valueOf(input.trim()).intValue();

      if (size <= 0)
         monitor.display("Invalid range ...", true);

      bufferSize = size;

      for (Enumeration e = dataSetVector.elements(); e.hasMoreElements(); ) {
        DSPlane data = (DSPlane)e.nextElement();
        data.setBufferSize(size);
      }
 
  }
 
  /**
   * animation speed change action
   */
  public void doAnimationSpeedChange(int speed) {
     if (speed < 10) animationSpeed = 10;
     else if (speed > 500) animationSpeed = 500;
     else animationSpeed = speed;

     if (animator == null) return;
     if (animator.isAlive()) {
        animator.stop();
        animator = null;
        animator = new Animator(this, animationSpeed);
        animator.start();
     }
  }


  /**
   * send window size change notification to all whiteboard object
   */
  public void changeWinSize(Dimension d) {
     for (Enumeration e=DHV.elements(); e.hasMoreElements();) {
       DrawHandle dh = (DrawHandle)e.nextElement();
       dh.setWindowSize(d);
     }
  }



  /**
   * make drawing object to string
   */
  public String getDHVString() {
    if (DHV.size() < 1) return null;

    StringBuffer str = new StringBuffer();      // return value buffer
    
    for (Enumeration e=DHV.elements(); e.hasMoreElements();) {
      DrawHandle dh = (DrawHandle)e.nextElement();

      if (dh.drawingVec.size() < 1) break;
      
      str.append(String.valueOf(dh.getTime() + "@"));
      Dimension sz = dh.getOrigWinSize();
      str.append(String.valueOf(sz.width + "@"));
      str.append(String.valueOf(sz.height + "@"));
      
      for (int i = 0; i < dh.drawingVec.size(); i++) {
         ShapeObject s = (ShapeObject)dh.drawingVec.elementAt(i);
         str.append(s.makeString() + "@");
      }
      str.append("%");       // deliminator among each time
    }

    return(new String(str));
  }


  /**
   * create drawing objects from received string
   * IT WILL BE CALLED FROM COLLABORATION MANAGER ONLY
   */
  public void createWhiteboard(String str) {
     if (str == null) return;

     StringTokenizer st = new StringTokenizer(str, "%");
     int count = st.countTokens();

     // replace current drawing handle vector
     DHV = new Vector();
     for (int i=0; i<count; i++) 
        DHV.addElement(new DrawHandle(st.nextToken()));

     for (Enumeration e=DHV.elements(); e.hasMoreElements();) {
       DrawHandle dh = (DrawHandle)e.nextElement();
       if (dh.getTime() == timeArr[currentTime]) {
          canvas.dh = dh;
          break;
       }
     }
  }


  // write all objects of this frame to a file
  public abstract void writeToFile(String fname);

  // make menu system
  protected abstract void makeMenu();

  // add menuItem to menu
  protected void addMI(String str, Menu parent, CSPlane command) {
     MenuItem mi = new MenuItem(str);
     mi.addActionListener(command);
     parent.add(mi);
  }

}  // end of class

