package svlight;

import java.applet.Applet;
import java.io.*;
import java.net.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

import svlight.kernel.*;
import svlight.ui.*;

public class SVClient extends Applet implements Runnable, ActionListener {

  Thread client_thread = null;

  DataInputStream sv_input = null;
  DataOutputStream sv_output = null;
  String serverName;
  int serverPort;

  public static FrameManager fmanager;

  public boolean c_mode = false;           // collaboration mode

  public void init() {
    try {
       Socket sock = new Socket(getCodeBase().getHost(), 7670);
       
       DataInputStream input = new DataInputStream(sock.getInputStream());
       DataOutputStream output = new DataOutputStream(sock.getOutputStream());
 
       output.writeInt(1);
       int new_port = input.readInt();
System.out.println("Port number is " + new_port);

       if (new_port < 0) {
          System.out.println("Exceed maximum allowed client, try again later.");
          return;
       }
       
       serverName = getCodeBase().getHost();
       serverPort = new_port + 10;
       sock = new Socket(getCodeBase().getHost(), new_port);
       sv_input = new DataInputStream(new BufferedInputStream(sock.getInputStream()));
       sv_output = new DataOutputStream(sock.getOutputStream());

       sv_output.writeInt(0);
       String sign = sv_input.readUTF();

       if (sign.indexOf("ok") >= 0) {
          fmanager = new FrameManager(this);
          makeGUI(getCodeBase().getHost(), new_port);

       } else {
System.out.println("Error: Connection to server was refused.");         
       }
     

    } catch (Exception e) {
       e.printStackTrace();
    }
  }

  /**
   * start thread
   */   
  public void start() {
     if (client_thread == null) {
        client_thread = new Thread(this, "Scivis Client");
        client_thread.start();
     }
  }


  /**
   * stop thread
   */
  public void stop() {
    System.out.println("Client will be stop.");
    try {
       sv_output.writeInt(-1);

       if (sv_input != null) sv_input.close();
       if (sv_output != null) sv_output.close();
    } catch (Exception e) {
       System.out.println("Closing client socket error.");
    }
   
    if (client_thread != null) {
      client_thread.stop();
      client_thread = null;
    }
  }

  /**
   * running thread
   */
  public void run() {
    
    try {

       while (true) {
          int command = sv_input.readInt();

          switch (command) {
              case 1:
System.out.println("Received 1d time data");
                doCommand_1();
                break;
              case 2:
System.out.println("Received multi-1d time data");
                doCommand_2();
                break;
              case 11:
System.out.println("Received 2d time data");
                doCommand_11();
                break;
              case 12:
System.out.println("Received multi-2d time data");
                doCommand_12();
                break;
              case 101:
System.out.println("Collaboration information received.");
               doCommand_101();
               break;
             
          }
       }
    } catch (Exception e) {
    }
  }

  // private variables for the GUI components

  private void makeGUI(String host_name, int port_num) {
     
     setLayout(new BorderLayout());

     BorderPanel bp = new BorderPanel();
     bp.setLayout(new GridLayout(2, 4, 1, 1));

     MediaTracker tracker = new MediaTracker(this);
     
     Image img[] = new Image[16];
     img[0] = getImage(getCodeBase(), "svlight/icons/connect.gif");
     img[1] = getImage(getCodeBase(), "svlight/icons/bolt.gif");
     img[2] = getImage(getCodeBase(), "svlight/icons/editor.gif");
     img[3] = getImage(getCodeBase(), "svlight/icons/help.gif");
     img[4] = getImage(getCodeBase(), "svlight/icons/recyc.gif");
     img[5] = getImage(getCodeBase(), "svlight/icons/no.gif");
     img[6] = getImage(getCodeBase(), "svlight/icons/graph.gif");
     img[7] = getImage(getCodeBase(), "svlight/icons/exit.gif");

     try {
        for (int i=0; i<8; i++) 
           tracker.addImage(img[i], i);

        for (int i=0; i<8; i++) 
           tracker.waitForID(i);
     } catch (Exception e) {}

     BarButton b;
     bp.add(b = new BarButton(img[0], img[0], "Connect"));
     b.addActionListener(this);
     bp.add(b = new BarButton(img[1], img[1], "Draw"));
     b.addActionListener(this);
     bp.add(b = new BarButton(img[2], img[2], "Editor"));
     b.addActionListener(this);
     bp.add(b = new BarButton(img[3], img[3], "Guide"));
     b.addActionListener(this);
     bp.add(b = new BarButton(img[4], img[4], "Kill All"));
     b.addActionListener(this);
     bp.add(b = new BarButton(img[5], img[5], "Check All"));
     b.addActionListener(this);
     bp.add(b = new BarButton(img[6], img[6], "Server"));
     b.addActionListener(this);
     bp.add(b = new BarButton(img[7], img[7], "Close"));
     b.addActionListener(this);

     display("Server: " +  host_name + ", " + "Port: " + port_num);

     add("Center", bp);
     validate();
  }

  /** 
   * do work when icon was clicked
   * @param e ActionEvent
   */
  public void actionPerformed(ActionEvent e) {
     String command = e.getActionCommand();

     if (e.getID() == MouseEvent.MOUSE_ENTERED) {
        if (command.equals("Connect")) { 
           message("Connect to other server for collaboration.", false);
        } else if (command.equals("Close")) {
           message("Close connection to the server.", false);
        } else if (command.equals("Guide")) {
           message("Help sector of this system.", false);
        } else if (command.equals("Kill All")) {
           message("Kill all visualization windows.", false);
        } else if (command.equals("Check All")) {
           message("Toggle to send command to all windows.", false);
        } else if (command.equals("Server")) {
           message("Get the server information.", false);
        } else if (command.equals("Editor")) {
           message("Filter creation wizard.", false);
        } else if (command.equals("Draw")) {
           message("Drawing tool.", false);
        } 
     } else if (e.getID() == MouseEvent.MOUSE_RELEASED) {
        if (command.equals("Connect")) { 
           if (c_mode) {           // collaboration mode("disconnect ..")
             SimpleDisplay sd = new SimpleDisplay(this);
             sd.setSize(200, 100);
             sd.show();           

           } else {                // not collaboration mode
             sendCommand(1, null);
	   }

        } else if (command.equals("Close")) {
	   display("Connection to the server, CLOSED!!!");
           try {
              sv_output.writeInt(-1);

           } catch (Exception E) {
              System.out.println("Closing client socket error.");
           }

        } else if (command.equals("Guide")) {
	   display("Only work with Scivis application version.");
        } else if (command.equals("Kill All")) {
	   display("Only work with Scivis application version.");
        } else if (command.equals("Check All")) {
	   display("Only work with Scivis application version.");
        } else if (command.equals("Server")) {
             SimpleDisplay sd = new SimpleDisplay(serverName, serverPort);
             sd.setSize(200, 120);
             sd.show();    
        } else if (command.equals("Editor")) {
	   display("Only work with Scivis application version.");
        } else if (command.equals("Draw")) {
	   display("Only work with Scivis application version.");
        } 
     }
  }

  /**
   * display meaage 
   */
  public void display(String str) {
     showStatus(str);
  }

  /**
   * display meaage 
   */
  public void message(String str, boolean b) {
     showStatus(str);
  }


  /**
   * send the command to the server
   */
  public void sendCommand(int command, String[] contents) {
     try {
        sv_output.writeInt(1);
        sv_output.writeInt(command);
        if (contents == null) 
           sv_output.writeUTF("null");
        else {
           for (int i=0; i<contents.length; i++)
              sv_output.writeUTF(contents[i]);
        }
 
     } catch (Exception e) {
        System.out.println("Sending command error.");
     }

  }


   //////////////////////////////////////////////////////////////
   // collaboration information
   private void doCommand_101() {
     try {
        int sz = sv_input.readInt();
        if (sz < 1) {
           SimpleDisplay sd = new SimpleDisplay();
           sd.setSize(320, 100);
           sd.show();

        } else { 
           String names[] = new String[sz];
           for (int i=0; i<sz; i++)
              names[i] = sv_input.readUTF();        
           CollSelector sel = new CollSelector(this, names);
           sel.setSize(240, 280);
           sel.show();
        }

     } catch (Exception e) {
        System.out.println("Data receiving error(101).");
     }
   }

   //////////////////////////////////////////////////////////////
   // for java_ser related
   private void doCommand_1() {
       TimeData1D td = parse1d();
       if (td == null) return;

       DataSet1D dataset = new DataSet1D(td.getTitle(), td);
       fmanager.receivedDataSet(dataset, null, -1, true);
   }

   //////////////////////////////////////////////////////////////
   // for java_mser related
   private void doCommand_2() {
       int sz;
       try {
         sz = sv_input.readInt();       
       } catch (Exception e) { 
         System.out.println("Vector size unknown(1d) ...");
         return;
       }

       Vector vec = new Vector();

       for (int i=0; i<sz; i++) {
          TimeData1D td = parse1d();
          if (td == null) {
             System.out.println("Time data receiving error(1d).");
             return;
          }
          vec.addElement(td);
       }
       TimeData1D td = (TimeData1D)vec.firstElement();
       if (td == null) return;

       DataSet1D dataset = new DataSet1D(td.getTitle(), vec);
       fmanager.receivedDataSet(dataset, null, -1, true);
   }

   //////////////////////////////////////////////////////////////
   // for java_ser2d related
   private void doCommand_11() {
       TimeData2D td = parse2d();
       if (td == null) return;

       DataSet2D dataset = new DataSet2D(td.getTitle(), td);
       fmanager.receivedDataSet(dataset, null, -1, true);
   }


   //////////////////////////////////////////////////////////////
   // for java_mser2d related
   private void doCommand_12() {
       int sz;
       try {
         sz = sv_input.readInt();       
       } catch (Exception e) { 
         System.out.println("Vector size unknown(2d) ...");
         return;
       }

       Vector vec = new Vector();

       for (int i=0; i<sz; i++) {
          TimeData2D td = parse2d();
          if (td == null) {
             System.out.println("Time data receiveing error(2d).");
             return;
          }
          vec.addElement(td);
       }
       TimeData2D td = (TimeData2D)vec.firstElement();
       if (td == null) return;

       DataSet2D dataset = new DataSet2D(td.getTitle(), vec);
       fmanager.receivedDataSet(dataset, null, -1, true);
   }

   ////////////////////////////////////////////////////////////
   // parse routines
   private TimeData1D parse1d() {
     try {
       TimeData1D td;
       boolean COLOR_CODE = false;
       boolean BB = false;
   
       float[] xArr = null, yArr = null, cArr = null;
       float xmin=0f, xmax=0f;

       // received time
       float time = sv_input.readFloat();

       String title = sv_input.readUTF();
       long serial = sv_input.readLong();

       // number of points in given time and make arrays
       int noOfPoints = sv_input.readInt();

       // received cArr
       if (sv_input.readByte() == 1) {
          COLOR_CODE = true;
          cArr = new float[noOfPoints];
          for (int i=0; i<noOfPoints; i++) 
             cArr[i] = sv_input.readFloat();
       } 

       // receive reserved 2 items and flush out
       sv_input.readInt();
       sv_input.readInt();
       
       // received x and y arrays
       if (sv_input.readByte() == 1) {
          BB = true;
          yArr = new float[noOfPoints];
        
          xmin = sv_input.readFloat();
          xmax = sv_input.readFloat();

          for (int i=0; i<noOfPoints; i++) 
             yArr[i] = sv_input.readFloat();

       } else {
          xArr = new float[noOfPoints];
          yArr = new float[noOfPoints];

          for (int i=0; i<noOfPoints; i++) {
             xArr[i] = sv_input.readFloat();
             yArr[i] = sv_input.readFloat();
          }
       }

       // SHOULD BE IMPLEMENTED FOR VECTOR_INCLUDED
       if (BB) {
         if (COLOR_CODE) 
           return null;   // not implemented yet
         else 
           td = new TimeData1D(time, noOfPoints, xmin, xmax, yArr);

       } else {

         if (COLOR_CODE) 
           td =  new TimeData1D(time, noOfPoints, xArr, yArr, cArr); 
         else 
           td =  new TimeData1D(time, noOfPoints, xArr, yArr);
       }
       td.setTitle(title);
       td.setserialNumber(serial);
       return td;

     } catch (Exception e) {
        System.out.println("Data receiving error(1).");
        return null;
     }

   }


   private TimeData2D parse2d() {
     try {
       TimeData2D td;
       boolean COLOR_CODE = false;
       boolean VECTOR_INCLUDED = false;     // not used yet
       boolean BB = false;
   
       //       TimeDataVec vectorSet = null;
       float[] xArr = null, yArr = null, zArr = null, cArr = null;
       float xmin=0f, ymin=0f, xmax=0f, ymax=0f;


       // received time
       float time = sv_input.readFloat();

       String title = sv_input.readUTF();
       long serial = sv_input.readLong();

       // number of points in given time and make arrays
       int nx = sv_input.readInt();
       int ny = sv_input.readInt();
       int noOfPoints = nx * ny;

       // received cArr
       if (sv_input.readByte() == 1) {
          COLOR_CODE = true;
          cArr = new float[noOfPoints];
          for (int i=0; i<noOfPoints; i++) 
             cArr[i] = sv_input.readFloat();
       } 

       // received vector set
       if (sv_input.readByte() == 1) {
         System.out.println("Vector data ...");
         return null;
	 //          vectorSet = parsevecStream(in);
	 //          VECTOR_INCLUDED = true;
       }

       // receive reserved 2 items and flush out
       sv_input.readInt();
       sv_input.readInt();       

       // received x, y, and z arrays
       if (sv_input.readByte() == 1) {
          BB = true;
          zArr = new float[noOfPoints];
        
          xmin = sv_input.readFloat();
          xmax = sv_input.readFloat();
          ymin = sv_input.readFloat();
          ymax = sv_input.readFloat();

          for (int i=0; i<noOfPoints; i++) 
             zArr[i] = sv_input.readFloat();

       } else {
          xArr = new float[noOfPoints];
          yArr = new float[noOfPoints];
          zArr = new float[noOfPoints];

          for (int i=0; i<noOfPoints; i++) {
             xArr[i] = sv_input.readFloat();
             yArr[i] = sv_input.readFloat();
             zArr[i] = sv_input.readFloat();
          }
       }

       // SHOULD BE IMPLEMENTED FOR VECTOR_INCLUDED
       if (BB) {
	 if (COLOR_CODE) 
           td = new TimeData2D(time, xmin, xmax, nx, 
                                      ymin, ymax, ny, zArr, cArr);
         else 
           td = new TimeData2D(time, xmin, xmax, nx, 
                                    ymin, ymax, ny, zArr);

       } else {
          // SHOULD BE IMPLEMENTED FOR COLOR_CODE
          td = new TimeData2D(time, noOfPoints, xArr, yArr, zArr);
       }

       td.setTitle(title);
       td.setserialNumber(serial);
       return td;

     } catch (Exception e) {
        System.out.println("Data receiving error(11).");
        return null;
     }

   }

} // end of class SVClient






