package svlight.ui;

import java.awt.*;
import java.awt.event.*;

public class BarButton extends Canvas {
    String label;
    String actionCommand;

    transient ActionListener actionListener;

    private Font font;
    private FontMetrics fm;
    private Image img, disabledImg;
    private boolean pressed;
    private boolean disabled = false;

    int wid = 70;
    int hgt = 55;

    public BarButton(Image img, Image other, String str) {

        font = new Font("Helvetica", Font.BOLD, 10);
        pressed = false;
        this.img = img;
        disabledImg = other;
        label = str;
 
        enableEvents(AWTEvent.MOUSE_EVENT_MASK);
        setSize(wid, hgt);
    }

    /**
     * Sets the command name of the action event fired by this button.
     * By default this will be set to the label of the button.
     */
    public void setActionCommand(String command) {
        actionCommand = command;
    }


    /**
     * Returns the command name of the action event fired by this button.
     */
    public String getActionCommand() {
        return (actionCommand == null? label : actionCommand);
    }

    /**
     * Adds the specified action listener to receive action events
     * from this button.
     * @param l the action listener
     */ 
    public synchronized void addActionListener(ActionListener l) {
	actionListener = AWTEventMulticaster.add(actionListener, l);
    }

    /**
     * Removes the specified action listener so it no longer receives
     * action events from this button.
     * @param l the action listener
     */ 
    public synchronized void removeActionListener(ActionListener l) {
	actionListener = AWTEventMulticaster.remove(actionListener, l);
    }


    public void paint(Graphics g) {

        if (fm == null) 
           fm = g.getFontMetrics(font);

        g.setColor(Color.lightGray);
        g.fill3DRect(0, 0, wid, hgt, !pressed);
      
        if (disabled) {
           g.drawImage(disabledImg, wid/2 - disabledImg.getWidth(this)/2, 5, this);
           g.setColor(Color.gray);
        } else {
           g.drawImage(img, wid/2 - img.getWidth(this)/2, 5, this);
           g.setColor(Color.black);
        }
        g.setFont(font);
        g.drawString(label, 
                       wid/2 - fm.stringWidth(label)/2,
		       img.getHeight(this) + fm.getMaxDescent() + 15);
    }

    public void update(Graphics g) {
        paint(g);
    }

   void highlight() {
      Graphics g = getGraphics();
      g.setColor(Color.gray);
      g.drawRect(0, 0, wid, hgt);
   }

   void unhighlight() {
      repaint();
   }

  // overrided function
  public void processEvent(AWTEvent e) {
     if (disabled) return;

     if (e.getID() == MouseEvent.MOUSE_PRESSED) {
        pressed = true;
        repaint();
     } else if (e.getID() == MouseEvent.MOUSE_RELEASED) {
        pressed = false;
        if (actionListener != null) {
           actionListener.actionPerformed(new ActionEvent(this, e.getID(), label));
        }
        repaint();
     } else if (e.getID() == MouseEvent.MOUSE_ENTERED) {
        if (actionListener != null) {
           actionListener.actionPerformed(new ActionEvent(this, e.getID(), label));
        }
        highlight();
     } else if (e.getID() == MouseEvent.MOUSE_EXITED) {
        unhighlight();
     }
     super.processEvent(e);
   }


  public void setDisabled(boolean b) {
      disabled = b;
      repaint();
  }

} // end of class



