package svlight.web;

import java.net.*;
import java.io.*;
import java.util.*;

import svlight.kernel.*;

class ClientManagerWorker extends Thread {
   private Socket sock;
   private DataInputStream in;

   ClientManager parent;

   /**
    * constructor
    * @param s server socket
    */
   public ClientManagerWorker(Socket s, ClientManager parent) {
      sock = s;
      this.parent = parent;
   }

   /**
    * read commands from the client and do work
    */
   public void run() {
      try {
	in  = new DataInputStream(new BufferedInputStream(sock.getInputStream()));
        String hostname = sock.getInetAddress().getHostName();
        int command = in.readInt();

        /////////////////////////////////////////////////////////////////
        // Now we have to decide what commands and data are received 
        // from the client. Normally, after receiving data, connection will
        // be disconnected. 

        doServerWork(hostname, command); // do the server work
        sock.close();                    // close the socket

      } catch (Exception e) {
	e.printStackTrace();
      }
      stop();
   }


   //////////////////////////////////////////////////////////////
   // for java_ser
   private void doCommand_10(String hostname) {

      try {
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         float time = in.readFloat();
         int nx = in.readInt();

         float[] xArr = new float[nx];
         float[] yArr = new float[nx];

         for (int j=0; j<nx; j++) {
            xArr[j] = in.readFloat();
            yArr[j] = in.readFloat();
         }

         TimeData1D td = new TimeData1D(time, nx, xArr, yArr);
         td.setTitle(title);
         td.setserialNumber(System.currentTimeMillis());
         DataOutputStream fout = new DataOutputStream(
            new FileOutputStream("/tmp/scivisLight/" + td.getserialNumber()));
         td.sendingData2(fout);
         fout.close();

         // real sending ...         
         parent.server.gotTimeData(td, parent.port);

      } catch (Exception e) {
e.printStackTrace();
         System.out.println("Data receiving/sending error(10).");
         return;
      }

   }


   //////////////////////////////////////////////////////////////
   // from java_bbser
   private void doCommand_28(String hostname) {
      try {
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         float time = in.readFloat();
         int nx = in.readInt();
         
         float xmin = in.readFloat();
         float xmax = in.readFloat();

         float[] yArr = new float[nx];

         for (int j=0; j<nx; j++) 
            yArr[j] = in.readFloat();

         TimeData1D td = new TimeData1D(time, nx, xmin, xmax, yArr);
         td.setTitle(title);
         td.setserialNumber(System.currentTimeMillis());
         DataOutputStream fout = new DataOutputStream(
            new FileOutputStream("/tmp/scivisLight/" + td.getserialNumber()));
         td.sendingData2(fout);
         fout.close();

         // real sending ...         
         parent.server.gotTimeData(td, parent.port);

      } catch (Exception e) {
         System.out.println("Data receiving/sending error(28).");
         return;
      }
   }

   //////////////////////////////////////////////////////////////
   // from java_mser
   private void doCommand_12(String hostname) {
      Vector vec = new Vector();

      try {
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         int nt = in.readInt();
         for (int i=0; i<nt; i++) {
            float time = in.readFloat();
            int nx = in.readInt();

            float[] xArr = new float[nx];
            float[] yArr = new float[nx];

            for (int j=0; j<nx; j++) {
               xArr[j] = in.readFloat();
               yArr[j] = in.readFloat();
            }

            TimeData1D td = new TimeData1D(time, nx, xArr, yArr);
            td.setTitle(title);
            td.setserialNumber(System.currentTimeMillis());
            DataOutputStream fout = new DataOutputStream(
              new FileOutputStream("/tmp/scivisLight/" + td.getserialNumber()));
            td.sendingData2(fout);
            fout.close();
            vec.addElement(td);
         }

         // real sending ...         
         parent.server.gotDataSet1D(vec, parent.port);

      } catch (Exception e) {
         System.out.println("Data receiving/sending error(12).");
         return;
      }
   }

   //////////////////////////////////////////////////////////////
   // from java_bbmser
   private void doCommand_29(String hostname) {
      Vector vec = new Vector();

      try {
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         int nt = in.readInt();
         for (int i=0; i<nt; i++) {
            float time = in.readFloat();
            size = in.readInt();

            float xmin = in.readFloat();
            float xmax = in.readFloat();

            float[] yArr = new float[size];

            for (int j=0; j<size; j++) 
               yArr[j] = in.readFloat();

            TimeData1D td = new TimeData1D(time, size, xmin, xmax, yArr);
            td.setTitle(title);
            td.setserialNumber(System.currentTimeMillis());
            DataOutputStream fout = new DataOutputStream(
              new FileOutputStream("/tmp/scivisLight/" + td.getserialNumber()));
            td.sendingData2(fout);
            fout.close();
            vec.addElement(td);
         }

         // real sending ...         
         parent.server.gotDataSet1D(vec, parent.port);

      } catch (Exception e) {
         System.out.println("Data receiving/sending error(29).");
         return;
      }
   }



   //////////////////////////////////////////////////////////////
   // from java_ser2d 
   private void doCommand_15(String hostname) {
       try {  
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         float time = in.readFloat();

         int nx = in.readInt();
         float[] xArr = new float[nx];
         float[] yArr = new float[nx];
         float[] zArr = new float[nx];

         for (int j=0; j<nx; j++) {
             xArr[j] = in.readFloat();
             yArr[j] = in.readFloat();
             zArr[j] = in.readFloat();
          }
         
         TimeData2D td = new TimeData2D(time, nx, xArr, yArr, zArr);
         td.setTitle(title);
         td.setserialNumber(System.currentTimeMillis());
         DataOutputStream fout = new DataOutputStream(
            new FileOutputStream("/tmp/scivisLight/" + td.getserialNumber()));
         td.sendingData2(fout);
         fout.close();

         // real sending ...         
         parent.server.gotTimeData(td, parent.port);

      } catch (Exception e) {
         System.out.println("Data receiving error(15).");
         return;
      }
   }

   //////////////////////////////////////////////////////////////
   // from java_bbser2d
   private void doCommand_25(String hostname) {
       try {  
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         float time = in.readFloat();
         int nx = in.readInt();
         int ny = in.readInt();

         float xmin = in.readFloat();
         float xmax = in.readFloat();
         float ymin = in.readFloat();
         float ymax = in.readFloat();

         size = nx * ny;
         float[] zArr = new float[size];

         for (int j=0; j<size; j++) 
             zArr[j] = in.readFloat();

         TimeData2D td = new TimeData2D(time, xmin, xmax, nx, 
                                          ymin, ymax, ny, zArr); 
         td.setTitle(title);
         td.setserialNumber(System.currentTimeMillis());
         DataOutputStream fout = new DataOutputStream(
            new FileOutputStream("/tmp/scivisLight/" + td.getserialNumber()));
         td.sendingData2(fout);
         fout.close();

         // real sending ...         
         parent.server.gotTimeData(td, parent.port);

      } catch (Exception e) {
         System.out.println("Data receiving/sending error(25).");
         return;
      }
   }


   //////////////////////////////////////////////////////////////
   // from java_mser2d
   private void doCommand_16(String hostname) {
      Vector vec = new Vector();

      try {
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         int nt = in.readInt();

         for (int i=0; i<nt; i++) {
             float time = in.readFloat();

             int nx = in.readInt();
             float[] xArr = new float[nx];
             float[] yArr = new float[nx];
             float[] zArr = new float[nx];

             for (int j=0; j<nx; j++) {
                xArr[j] = in.readFloat();
                yArr[j] = in.readFloat();
                zArr[j] = in.readFloat();
             }


            TimeData2D td = new TimeData2D(time, nx, xArr, yArr, zArr);
            td.setTitle(title);
            td.setserialNumber(System.currentTimeMillis());
            DataOutputStream fout = new DataOutputStream(
              new FileOutputStream("/tmp/scivisLight/" + td.getserialNumber()));
            td.sendingData2(fout);
            fout.close();
            vec.addElement(td);
         }

         // real sending ...         
         parent.server.gotDataSet2D(vec, parent.port);

      } catch (Exception e) {
         System.out.println("Data receiving/sending error(16).");
         return;
      }
   }

   //////////////////////////////////////////////////////////////
   // from java_bbmser2d
   private void doCommand_26(String hostname) {
      Vector vec = new Vector();

      try {
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         int nt = in.readInt();

         for (int i=0; i<nt; i++) {
             float time = in.readFloat();

             int nx = in.readInt();
             int ny = in.readInt();

             float xmin = in.readFloat();
             float xmax = in.readFloat();
             float ymin = in.readFloat();
             float ymax = in.readFloat();

             size = nx * ny;
             float[] zArr = new float[size];

             for (int j=0; j<size; j++) {
                zArr[j] = in.readFloat();
             }

            TimeData2D td = new TimeData2D(time, xmin, xmax, nx, ymin, ymax, ny, zArr);
            td.setTitle(title);
            td.setserialNumber(System.currentTimeMillis());
            DataOutputStream fout = new DataOutputStream(
              new FileOutputStream("/tmp/scivisLight/" + td.getserialNumber()));
            td.sendingData2(fout);
            fout.close();
            vec.addElement(td);
         }

         // real sending ...         
         parent.server.gotDataSet2D(vec, parent.port);

      } catch (Exception e) {
         System.out.println("Data receiving/sending error(26).");
         return;
      }
   }


    
   // real worker
   private synchronized void doServerWork(String hostname, int command) {
System.out.println("Command = " + command);
     switch(command) {
        case 10:  doCommand_10(hostname);   break;
        case 12:  doCommand_12(hostname);   break;
        case 15:  doCommand_15(hostname);   break;
	case 16:  doCommand_16(hostname);   break;  
        case 25:  doCommand_25(hostname);   break; 
        case 26:  doCommand_26(hostname);   break;
        case 28:  doCommand_28(hostname);   break;
        case 29:  doCommand_29(hostname);   break;
       
     }
   }

} // end of class ClientManagerWorker



