package svlight.web;

import java.net.*;
import java.io.*;
import java.util.*;
import svlight.kernel.*;

public class SVWebServer implements Runnable {

  Thread runner;

  private static final int DEFAULT_PORT = 7670;
  private static final int SERVER_ALLOWED = 5;
  
  private ServerSocket mainSock = null;
  private WebManager[] managers;
  private ClientManager[] cmanagers;

  /**
   * constructor
   */
  public SVWebServer() {
     try {
        mainSock = new ServerSocket(DEFAULT_PORT);

        managers = new WebManager[SERVER_ALLOWED];
        cmanagers = new ClientManager[SERVER_ALLOWED];

        for (int i=0; i<SERVER_ALLOWED; i++) {
           managers[i] = new WebManager(this, DEFAULT_PORT+i+1);
           managers[i].start();
System.out.println("Listening at web Port " + (DEFAULT_PORT+i+1));
        }

        for (int i=0; i<SERVER_ALLOWED; i++) {
           cmanagers[i] = new ClientManager(this, DEFAULT_PORT+i+11);
           cmanagers[i].start();
System.out.println("Listening at client Port " + (DEFAULT_PORT+i+11));
        }        
        

     } catch(Exception e) { 
       //e.printStackTrace();
        System.err.println("\nCan't create system socket."); 
        System.exit(1);
     }  
  }

  ////////////////////////////////////////////////////////////////////////////
  //  COLLABORATION
  ////////////////////////////////////////////////////////////////////////////
  /**
   * return host names connected to this server
   */
  public String[] getHostNames(int p) {
     Vector vec = new Vector();
     p = p - DEFAULT_PORT - 1;

     for (int i=0; i<SERVER_ALLOWED; i++) {
        if (i == p) continue;
        if (managers[i].isValidServer()) 
           vec.addElement(managers[i].getClientName());
     }
     if (vec.size() < 1) return null;
     String[] hosts = new String[vec.size()];
     vec.copyInto(hosts);
     return hosts;
  }

  ////////////////////////////////////////////////////////////////////////////

  /**
   * send the 1d time data to the client
   */
  public void gotTimeData(TimeData1D td, int port_num) {
     int idx = port_num - (DEFAULT_PORT+11);
     managers[idx].sendTimeData(td);
  }

  /**
   * send the 2d time data to the client
   */
  public void gotTimeData(TimeData2D td, int port_num) {
     int idx = port_num - (DEFAULT_PORT+11);
     managers[idx].sendTimeData(td);
  }

  /**
   * send the 1d multi-time data to the client
   */
  public void gotDataSet1D(Vector vec, int port_num) {
     int idx = port_num - (DEFAULT_PORT+11);
     managers[idx].sendDataSet1D(vec);
  }

  /**
   * send the 2d multi-time data to the client
   */
  public void gotDataSet2D(Vector vec, int port_num) {
     int idx = port_num - (DEFAULT_PORT+11);
     managers[idx].sendDataSet2D(vec);
  }


  /**
   * waits for a connection then spawns a WebManager to deal with it
   */
  public void run() {
     DataOutputStream out = null;
     DataInputStream in = null;

     // continuously listen the socket
     while (true) {
        try {
           Socket sock_con = mainSock.accept();

           out = new DataOutputStream(sock_con.getOutputStream());
	   in  = new DataInputStream(new 
                        BufferedInputStream(sock_con.getInputStream()));

           int command = in.readInt();
	   //System.out.println("Got command: " + command);

           if (command < 0) {          // kill the server
	      for (int i=0; i<SERVER_ALLOWED; i++) {
                 managers[i].killServer();
                 cmanagers[i].killServer();
	      }

              sock_con.close();
              mainSock.close();
              System.out.println("ScivisLight server exits.");
              System.exit(0);
              
           } else {                    // connection from the client
              boolean SERVED = false;
              for (int i=0; i<SERVER_ALLOWED; i++) {
                 if (!managers[i].isValidServer()) {
                    managers[i].assginClient(sock_con.getInetAddress());
		    //System.out.println("Connection established.");
                    out.writeInt(DEFAULT_PORT+i+1);
                    SERVED = true;
                    break;
		 }
              }
              if (!SERVED) {
                 out.writeInt(-1);
	      }
           }
 
	
        } catch(Exception e) {
e.printStackTrace();
           System.err.println("Can't accept any connection from the client.");
           return;
        }
     }
  }


  /**
   * start thread
   */   
  public void start() {
    if (runner == null); {
      runner = new Thread(this);
      runner.start();
    }
  }

  /**
   * stop thread
   */
  public void stop() {
    if (runner != null) {
      runner.stop();
      runner = null;
    }
  }


  // main function
  public static void main(String[] args) {
     SVWebServer server = new SVWebServer();
     server.start();
  }

} // end of class SVWebServer



