package svlight.web;

import java.net.*;
import java.io.*;
import java.util.*;
import svlight.kernel.*;

class WebManager extends Thread {

   private ServerSocket mainSock = null;
   private Socket connection = null;

   private InetAddress client = null;
   private DataInputStream input = null;
   private DataOutputStream output = null;

   private boolean alive = false;
      
   SVWebServer server;
   int port;
   
   /**
    * constructor
    */
   public WebManager(SVWebServer server, int port) {
      this.server = server;
      this.port = port;

      try {
        mainSock = new ServerSocket(port);

      } catch(Exception e) { 
        System.err.println("\nCan't create socket for the web client."); 
     }  
   }

  /**
   * assign to work
   */
  public void assginClient(InetAddress client) {
    this.client = client;
    alive = true;
  }

  /** 
   * return this server is workring correctly
   */
  public boolean isValidServer() {
    return alive;
  }

  /**
   * return client's name
   */
  public String getClientName() {
     if (client.getHostName() == null) 
        return "localhost";
     else
        return client.getHostName();
  }

  /** 
   * send 1d time data
   */
  public void sendTimeData(TimeData1D td) {
System.out.println("sending 1d time data at port " + port);
     try {
        output.writeInt(1);
        td.sendingData2(output);
     } catch (Exception e) {
        System.out.println("Data sending error(1) ...");
     }
  }


  /** 
   * send 2d time data
   */
  public void sendTimeData(TimeData2D td) {
System.out.println("sending 2d time data at port " + port);
     try {
        output.writeInt(11);
        td.sendingData2(output);
     } catch (Exception e) {
        System.out.println("Data sending error(11) ...");
     }
  }


  /** 
   * send multi-1d time data
   */
  public void sendDataSet1D(Vector vec) {
System.out.println("sending multi-1d time data at port " + port);
     try {
        output.writeInt(2);
        output.writeInt(vec.size());
        for (int i=0; i<vec.size(); i++) {
          TimeData1D td = (TimeData1D)vec.elementAt(i);
          td.sendingData2(output);
        }
       
     } catch (Exception e) {
        System.out.println("Data sending error(2) ...");
     }
  }


  /** 
   * send multi-2d time data
   */
  public void sendDataSet2D(Vector vec) {
System.out.println("sending multi-2d time data at port " + port);
     try {
        output.writeInt(12);
        output.writeInt(vec.size());
        for (int i=0; i<vec.size(); i++) {
          TimeData2D td = (TimeData2D)vec.elementAt(i);
          td.sendingData2(output);
        }
     } catch (Exception e) {
        System.out.println("Data sending error(12) ...");
     }
  }


   /**
    * read commands from the client and do work
    */
   public void run() {
     try {
        while (true) {
           connection = mainSock.accept();
System.out.println("new connection ...");
           output = new DataOutputStream(connection.getOutputStream());
           input  = new DataInputStream(new BufferedInputStream(connection.getInputStream()));

           preserveConnection();
        }
     } catch(Exception e) {
e.printStackTrace();
        System.err.println("Can't accept client's request.");
        return;
     }

   }

  /**
   * kill this server
   */
  public void killServer() {
     try {
        if (input != null) input.close();
        if (output != null) output.close();
        if (mainSock != null) mainSock.close();
        if (connection != null) connection.close();

     } catch (Exception e) {
e.printStackTrace();
        System.out.println("Socket closing error.");
     }

     stop();
  }


  // send ok sign to client
  private void sendOk() {
    try {
       output.writeUTF("ok");
    } catch (Exception e) {
       System.out.println("Write ok sign error.");
    }
  }

  // preserver the connection
  private void preserveConnection() {
    try {
      while (true) {
        int command = input.readInt();
System.out.println("WebManager command: " + command);

        if (command == 0) {
           sendOk();

        } else if (command == 1) {
           execute();

        } else if (command == -1) {
           if (input != null) input.close();
           if (output != null) output.close();
           if (connection != null) connection.close();
           alive = false;
           return;
        } 

      }
    } catch (Exception e) {
e.printStackTrace();
       System.out.println("Socket may be out of date.");
    }
  }



  ////////////////////////////////////////////////////////////////
  // execute the server work
  private void execute() {
     try {
        int command = input.readInt();

        if (command == 1) {
	  // received collaboration request
          String hosts[] = server.getHostNames(port);  

          output.writeInt(101);
          if (hosts == null)  output.writeInt(0);
          else {    
             output.writeInt(hosts.length);
             for (int i=0; i<hosts.length; i++)
                output.writeUTF(hosts[i]);
	  }
        } else {
System.out.println("Unknown command.");
        }

     } catch (Exception e) {
e.printStackTrace();
       System.out.println("Server execution error caused by connection problem.");
     }
    }

} // end of class WebManager





